@@ -21,6 +21,8 @@
 #ifndef _MAINWINDOW_HPP_
 #define _MAINWINDOW_HPP_
 
+#include <gtkmm/applicationwindow.h>
+
 #include "mainwindowembeds.hpp"
 #include "note.hpp"
 
@@ -35,7 +37,6 @@ class MainWindow
   , public EmbeddableWidgetHost
 {
 public:
-  static MainWindow *get_owning(Gtk::Widget & widget);
   static void present_in(MainWindow & win, const Note::Ptr & note);
   static MainWindow *present_in_new_window(IGnote & g, const Note::Ptr & note);
   static MainWindow *present_default(IGnote & g, const Note::Ptr & note);
@@ -49,7 +50,6 @@ public:
   virtual void new_note() = 0;
   virtual void close_window() = 0;
   virtual bool is_search() = 0;
-  virtual utils::GlobalKeybinder & keybinder() = 0;
 protected:
   virtual void present_note(const Note::Ptr & note) = 0;
 private:
